# coding: utf-8

"""
    DataCore API  - Sport

    For more details, see openapi/init.py

    The version of the OpenAPI document: v1
    Generated by OpenAPI Generator (https://openapi-generator.tech)

    Do not edit the file manually.
"""  # noqa: E501


from __future__ import annotations

import json
import pprint
import re  # noqa: F401
from datetime import date, datetime
from typing import List, Optional

from pydantic import BaseModel, Field, StrictStr, conlist, constr, validator

from atriumsports.datacore.openapi.models.images_model import ImagesModel
from atriumsports.datacore.openapi.models.uniforms_model_organization import UniformsModelOrganization


class UniformsModel(BaseModel):
    """
    UniformsModel
    """

    uniform_id: Optional[StrictStr] = Field(None, alias="uniformId", description="The unique identifier of the Uniform")
    organization_id: Optional[StrictStr] = Field(
        None, alias="organizationId", description="The unique identifier of the organization"
    )
    organization: Optional[UniformsModelOrganization] = None
    base_type: Optional[constr(strict=True, max_length=100)] = Field(
        None,
        alias="baseType",
        description="The object that this uniform relates to >- `ENTITY` Entity >- `ENTITYGROUP` Entity Group >- `PERSON` Person ",
    )
    base_id: Optional[StrictStr] = Field(
        None,
        alias="baseId",
        description="The unique identifier of the object associated with this record. If the `baseType` is `ENTITY` then this would be the value of `entityId`.",
    )
    name_local: Optional[constr(strict=True, max_length=150)] = Field(
        None,
        alias="nameLocal",
        description="The name of the uniforms in the [local](#section/Introduction/Character-Sets-and-Names) language",
    )
    name_latin: Optional[constr(strict=True, max_length=150)] = Field(
        None,
        alias="nameLatin",
        description="The name of the uniforms in [latin](#section/Introduction/Character-Sets-and-Names) characters",
    )
    date_from: Optional[date] = Field(None, alias="dateFrom", description="Date the Uniform is valid from")
    date_to: Optional[date] = Field(None, alias="dateTo", description="Date the Uniform is valid until")
    external_id: Optional[constr(strict=True, max_length=150)] = Field(
        None, alias="externalId", description="The Id of the data as set by the provider of the data"
    )
    updated: Optional[datetime] = Field(None, description="Date/time last modified. In UTC")
    added: Optional[datetime] = Field(None, description="Date/time added. In UTC")
    images: Optional[conlist(ImagesModel)] = None
    __properties = [
        "uniformId",
        "organizationId",
        "organization",
        "baseType",
        "baseId",
        "nameLocal",
        "nameLatin",
        "dateFrom",
        "dateTo",
        "externalId",
        "updated",
        "added",
        "images",
    ]

    @validator("base_type")
    def base_type_validate_enum(cls, value):
        """Validates the enum"""
        if value is None:
            return value

        if value not in ("PERSON", "ENTITY", "ENTITYGROUP"):
            raise ValueError("must be one of enum values ('PERSON', 'ENTITY', 'ENTITYGROUP')")
        return value

    @validator("external_id")
    def external_id_validate_regular_expression(cls, value):
        """Validates the regular expression"""
        if value is None:
            return value

        if not re.match(r"^[0-9A-Za-z\/_-]+$", value):
            raise ValueError(r"must validate the regular expression /^[0-9A-Za-z\/_-]+$/")
        return value

    class Config:
        """Pydantic configuration"""

        allow_population_by_field_name = True
        validate_assignment = True

    def to_str(self) -> str:
        """Returns the string representation of the model using alias"""
        return pprint.pformat(self.dict(by_alias=True))

    def to_json(self) -> str:
        """Returns the JSON representation of the model using alias"""
        return json.dumps(self.to_dict())

    @classmethod
    def from_json(cls, json_str: str) -> UniformsModel:
        """Create an instance of UniformsModel from a JSON string"""
        return cls.from_dict(json.loads(json_str))

    def to_dict(self):
        """Returns the dictionary representation of the model using alias"""
        _dict = self.dict(
            by_alias=True,
            exclude={
                "organization_id",
                "updated",
                "added",
            },
            exclude_none=True,
        )
        # override the default output from pydantic by calling `to_dict()` of organization
        if self.organization:
            _dict["organization"] = self.organization.to_dict()
        # override the default output from pydantic by calling `to_dict()` of each item in images (list)
        _items = []
        if self.images:
            for _item in self.images:
                if _item:
                    _items.append(_item.to_dict())
            _dict["images"] = _items
        # set to None if name_latin (nullable) is None
        # and __fields_set__ contains the field
        if self.name_latin is None and "name_latin" in self.__fields_set__:
            _dict["nameLatin"] = None

        # set to None if date_from (nullable) is None
        # and __fields_set__ contains the field
        if self.date_from is None and "date_from" in self.__fields_set__:
            _dict["dateFrom"] = None

        # set to None if date_to (nullable) is None
        # and __fields_set__ contains the field
        if self.date_to is None and "date_to" in self.__fields_set__:
            _dict["dateTo"] = None

        # set to None if external_id (nullable) is None
        # and __fields_set__ contains the field
        if self.external_id is None and "external_id" in self.__fields_set__:
            _dict["externalId"] = None

        return _dict

    @classmethod
    def from_dict(cls, obj: dict) -> UniformsModel:
        """Create an instance of UniformsModel from a dict"""
        if obj is None:
            return None

        if not isinstance(obj, dict):
            return UniformsModel.parse_obj(obj)

        _obj = UniformsModel.parse_obj(
            {
                "uniform_id": obj.get("uniformId"),
                "organization_id": obj.get("organizationId"),
                "organization": UniformsModelOrganization.from_dict(obj.get("organization"))
                if obj.get("organization") is not None
                else None,
                "base_type": obj.get("baseType"),
                "base_id": obj.get("baseId"),
                "name_local": obj.get("nameLocal"),
                "name_latin": obj.get("nameLatin"),
                "date_from": obj.get("dateFrom"),
                "date_to": obj.get("dateTo"),
                "external_id": obj.get("externalId"),
                "updated": obj.get("updated"),
                "added": obj.get("added"),
                "images": [ImagesModel.from_dict(_item) for _item in obj.get("images")]
                if obj.get("images") is not None
                else None,
            }
        )
        return _obj
